use log::{debug, warn};
use peach_lib::config_manager;
use peach_network::network;
use rouille::{post_input, try_or_400, Request, Response};

use crate::auth;
use crate::context;
use crate::templates;

pub fn menu() -> Response {
    debug!("received GET request for: /settings");

    Response::html(templates::settings::menu::menu())
}

pub fn admin() -> Response {
    debug!("received GET request for: /settings/admin");

    Response::html(templates::settings::admin::menu())
}

pub fn admin_configure() -> Response {
    debug!("received GET request for: /settings/admin/configure");

    Response::html(templates::settings::admin::configure(None, None))
}

pub fn admin_add() -> Response {
    debug!("received GET request for: /settings/admin/add");

    Response::html(templates::settings::admin::add(None, None))
}

pub fn admin_add_post(request: &Request) -> Response {
    debug!("received POST request for: /settings/admin/add");

    let data = try_or_400!(post_input!(request, { ssb_id: String }));

    debug!("{:?}", data);

    let (flash_name, flash_msg) = match config_manager::add_ssb_admin_id(&data.ssb_id) {
        Ok(_) => (
            "success".to_string(),
            "Added new SSB administrator ID".to_string(),
        ),
        Err(e) => (
            "error".to_string(),
            format!("Failed to add new SSB administrator ID: {}", e),
        ),
    };

    Response::html(templates::settings::admin::add(
        Some(flash_msg),
        Some(flash_name),
    ))
}

pub fn admin_change_password() -> Response {
    debug!("received GET request for: /settings/admin/change_password");

    Response::html(templates::settings::admin::change_password(None, None))
}

pub fn admin_change_password_post(request: &Request) -> Response {
    debug!("received POST request for: /settings/admin/change_password");

    let data = try_or_400!(post_input!(request, {
        current_password: String,
        new_password1: String,
        new_password2: String
    }));

    // attempt to update the password
    let (flash_name, flash_msg) = match auth::save_password_form(
        data.current_password,
        data.new_password1,
        data.new_password2,
    ) {
        Ok(_) => ("success".to_string(), "Saved new password".to_string()),
        Err(e) => (
            "error".to_string(),
            format!("Failed to save new password: {}", e),
        ),
    };

    Response::html(templates::settings::admin::change_password(
        Some(flash_msg),
        Some(flash_name),
    ))
}

pub fn admin_delete_post(request: &Request) -> Response {
    debug!("received POST request for: /settings/admin/delete");

    let data = try_or_400!(post_input!(request, { ssb_id: String }));

    let (flash_name, flash_msg) = match config_manager::delete_ssb_admin_id(&data.ssb_id) {
        Ok(_) => (
            "success".to_string(),
            "Removed SSB administrator ID".to_string(),
        ),
        Err(e) => (
            "error".to_string(),
            format!("Failed to remove SSB administrator ID: {}", e),
        ),
    };

    Response::html(templates::settings::admin::configure(
        Some(flash_msg),
        Some(flash_name),
    ))
}

pub fn network() -> Response {
    debug!("received GET request for: /settings/network");

    Response::html(templates::settings::network::menu())
}

pub fn network_add_ap(ssid: Option<String>) -> Response {
    debug!("received GET request for: /settings/network/wifi/add");

    Response::html(templates::settings::network::add_ap(ssid, None, None))
}

pub fn network_add_ap_post(request: &Request) -> Response {
    debug!("received POST request for: /settings/network/wifi/add");

    // TODO: read this value from the config file instead
    let wlan_iface = "wlan0".to_string();

    let data = try_or_400!(post_input!(request, { ssid: String, pass: String }));

    /* ADD WIFI CREDENTIALS FOR AP */

    // check if the credentials already exist for this access point
    let creds_exist = match network::saved_networks() {
        Ok(Some(ssids)) => ssids.contains(&data.ssid),
        _ => false,
    };

    let (flash_name, flash_msg) = if creds_exist {
        (
            "error".to_string(),
            "Network credentials already exist for this access point".to_string(),
        )
    } else {
        // if credentials not found, generate and write wifi config to wpa_supplicant
        match network::add(&wlan_iface, &data.ssid, &data.pass) {
            Ok(_) => {
                debug!("Added WiFi credentials.");
                // force reread of wpa_supplicant.conf file with new credentials
                match network::reconfigure() {
                    Ok(_) => debug!("Successfully reconfigured wpa_supplicant"),
                    Err(_) => warn!("Failed to reconfigure wpa_supplicant"),
                }
                ("success".to_string(), "Added WiFi credentials".to_string())
            }
            Err(e) => {
                debug!("Failed to add WiFi credentials.");
                ("error".to_string(), format!("{}", e))
            }
        }
    };

    Response::html(templates::settings::network::add_ap(
        None,
        Some(flash_msg),
        Some(flash_name),
    ))
}

pub fn network_configure_dns() -> Response {
    debug!("received GET request for: /settings/network/dns");

    Response::html(templates::settings::network::configure_dns(None, None))
}

pub fn network_configure_dns_post(request: &Request) -> Response {
    debug!("received POST request for: /settings/network/dns");

    let data = try_or_400!(
        post_input!(request, { external_domain: String, enable_dyndns: bool, dynamic_domain: String })
    );

    let (flash_name, flash_msg) = match context::network::save_dns_configuration(
        data.external_domain,
        data.enable_dyndns,
        data.dynamic_domain,
    ) {
        Ok(_) => (
            "success".to_string(),
            "New dynamic DNS configuration is now enabled".to_string(),
        ),
        Err(e) => (
            "error".to_string(),
            format!("Failed to save DNS configuration: {}", e),
        ),
    };

    Response::html(templates::settings::network::configure_dns(
        Some(flash_msg),
        Some(flash_name),
    ))
}

pub fn network_modify_ap(ssid: Option<String>) -> Response {
    debug!("received GET request for: /settings/network/wifi/modify");

    Response::html(templates::settings::network::modify_ap(ssid, None, None))
}

pub fn network_modify_ap_post(request: &Request) -> Response {
    debug!("received POST request for: /settings/network/wifi/modify");

    // TODO: read this value from the config file instead
    let wlan_iface = "wlan0".to_string();

    let data = try_or_400!(post_input!(request, { ssid: String, pass: String }));

    /* MODIFY WIFI CREDENTIALS FOR AP */
    let (flash_name, flash_msg) = match network::update(&wlan_iface, &data.ssid, &data.pass) {
        Ok(_) => ("success".to_string(), "WiFi password updated".to_string()),
        Err(e) => (
            "error".to_string(),
            format!("Failed to update WiFi password: {}", e),
        ),
    };

    Response::html(templates::settings::network::modify_ap(
        None,
        Some(flash_msg),
        Some(flash_name),
    ))
}

pub fn network_connect_wifi(request: &Request) -> Response {
    debug!("received POST request for: /settings/network/wifi/connect");

    // TODO: read this value from the config file instead
    let wlan_iface = "wlan0".to_string();

    let data = try_or_400!(post_input!(request, { ssid: String }));

    let (flash_name, flash_msg) = match network::id(&wlan_iface, &data.ssid) {
        Ok(Some(id)) => match network::connect(&id, &wlan_iface) {
            Ok(_) => (
                "success".to_string(),
                "Connected to chosen network".to_string(),
            ),
            Err(e) => (
                "error".to_string(),
                format!("Failed to connect to chosen network: {}", e),
            ),
        },

        _ => (
            "error".to_string(),
            "Failed to retrieve the network ID".to_string(),
        ),
    };

    Response::html(templates::settings::network::network_detail(
        data.ssid,
        Some(flash_msg),
        Some(flash_name),
    ))
}

pub fn network_disconnect_wifi(request: &Request) -> Response {
    debug!("received POST request for: /settings/network/wifi/disconnect");

    // TODO: read this value from the config file instead
    let wlan_iface = "wlan0".to_string();

    let data = try_or_400!(post_input!(request, { ssid: String }));

    let (flash_name, flash_msg) = match network::disable(&wlan_iface, &data.ssid) {
        Ok(_) => (
            "success".to_string(),
            "Disconnected from WiFi network".to_string(),
        ),
        Err(e) => (
            "error".to_string(),
            format!("Failed to disconnect from WiFi network: {}", e),
        ),
    };

    Response::html(templates::settings::network::network_detail(
        data.ssid,
        Some(flash_msg),
        Some(flash_name),
    ))
}

pub fn network_forget_wifi(request: &Request) -> Response {
    debug!("received POST request for: /settings/network/wifi/forget");

    // TODO: read this value from the config file instead
    let wlan_iface = "wlan0".to_string();

    let data = try_or_400!(post_input!(request, { ssid: String }));

    let (flash_name, flash_msg) = match network::forget(&wlan_iface, &data.ssid) {
        Ok(_) => (
            "success".to_string(),
            "WiFi credentials removed".to_string(),
        ),
        Err(e) => (
            "error".to_string(),
            format!("Failed to remove WiFi credentials: {}", e),
        ),
    };

    Response::html(templates::settings::network::network_detail(
        data.ssid,
        Some(flash_msg),
        Some(flash_name),
    ))
}

pub fn network_detail(ssid: String) -> Response {
    debug!("received GET request for: /settings/network/wifi/<selected>");

    Response::html(templates::settings::network::network_detail(
        ssid, None, None,
    ))
}

pub fn network_list_aps() -> Response {
    debug!("received GET request for: /settings/network/wifi");

    Response::html(templates::settings::network::list_aps())
}

pub fn scuttlebutt() -> Response {
    debug!("received GET request for: /settings/scuttlebutt");

    Response::html(templates::settings::scuttlebutt::scuttlebutt())
}
